@@ -8,34 +8,82 @@
 -- Portability : unknown
 --
 module Network.TLS.Cipher
-    ( BulkFunctions(..)
-    , CipherKeyExchangeType(..)
+    ( CipherKeyExchangeType(..)
     , Bulk(..)
+    , BulkFunctions(..)
+    , BulkDirection(..)
+    , BulkState(..)
+    , BulkStream(..)
+    , BulkBlock
+    , BulkAEAD
+    , bulkInit
     , Hash(..)
     , Cipher(..)
     , CipherID
-    , Key
-    , IV
     , cipherKeyBlockSize
+    , BulkKey
+    , BulkIV
+    , BulkNonce
+    , BulkAdditionalData
     , cipherAllowedForVersion
     , cipherExchangeNeedMoreData
+    , hasMAC
+    , hasRecordIV
     ) where
 
+import Crypto.Cipher.Types (AuthTag)
 import Network.TLS.Types (CipherID)
 import Network.TLS.Struct (Version(..))
+import Network.TLS.Crypto (Hash(..), hashDigestSize)
 
 import qualified Data.ByteString as B
 
 -- FIXME convert to newtype
-type Key = B.ByteString
-type IV = B.ByteString
+type BulkKey = B.ByteString
+type BulkIV = B.ByteString
+type BulkNonce = B.ByteString
+type BulkAdditionalData = B.ByteString
+
+data BulkState =
+      BulkStateStream BulkStream
+    | BulkStateBlock  BulkBlock
+    | BulkStateAEAD   BulkAEAD
+    | BulkStateUninitialized
+
+instance Show BulkState where
+    show (BulkStateStream _)      = "BulkStateStream"
+    show (BulkStateBlock _)       = "BulkStateBlock"
+    show (BulkStateAEAD _)        = "BulkStateAEAD"
+    show (BulkStateUninitialized) = "BulkStateUninitialized"
+
+newtype BulkStream = BulkStream (B.ByteString -> (B.ByteString, BulkStream))
+
+type BulkBlock = BulkIV -> B.ByteString -> (B.ByteString, BulkIV)
+
+type BulkAEAD = BulkNonce -> B.ByteString -> BulkAdditionalData -> (B.ByteString, AuthTag)
+
+data BulkDirection = BulkEncrypt | BulkDecrypt
+    deriving (Show,Eq)
+
+bulkInit :: Bulk -> BulkDirection -> BulkKey -> BulkState
+bulkInit bulk direction key =
+    case bulkF bulk of
+        BulkBlockF  ini -> BulkStateBlock  (ini direction key)
+        BulkStreamF ini -> BulkStateStream (ini direction key)
+        BulkAeadF   ini -> BulkStateAEAD   (ini direction key)
 
 data BulkFunctions =
-      BulkBlockF (Key -> IV -> B.ByteString -> B.ByteString)
-                 (Key -> IV -> B.ByteString -> B.ByteString)
-    | BulkStreamF (Key -> IV)
-                  (IV -> B.ByteString -> (B.ByteString, IV))
-                  (IV -> B.ByteString -> (B.ByteString, IV))
+      BulkBlockF  (BulkDirection -> BulkKey -> BulkBlock)
+    | BulkStreamF (BulkDirection -> BulkKey -> BulkStream)
+    | BulkAeadF   (BulkDirection -> BulkKey -> BulkAEAD)
+
+hasMAC,hasRecordIV :: BulkFunctions -> Bool
+
+hasMAC (BulkBlockF _ ) = True
+hasMAC (BulkStreamF _) = True
+hasMAC (BulkAeadF _  ) = False
+
+hasRecordIV = hasMAC
 
 data CipherKeyExchangeType =
       CipherKeyExchange_RSA
@@ -54,6 +102,8 @@ data Bulk = Bulk
     { bulkName         :: String
     , bulkKeySize      :: Int
     , bulkIVSize       :: Int
+    , bulkExplicitIV   :: Int -- Explicit size for IV for AEAD Cipher, 0 otherwise
+    , bulkAuthTagLen   :: Int -- Authentication tag length in bytes for AEAD Cipher, 0 otherwise
     , bulkBlockSize    :: Int
     , bulkF            :: BulkFunctions
     }
@@ -67,17 +117,6 @@ instance Eq Bulk where
                    , bulkBlockSize b1 == bulkBlockSize b2
                    ]
 
-data Hash = Hash
-    { hashName         :: String
-    , hashSize         :: Int
-    , hashF            :: B.ByteString -> B.ByteString
-    }
-
-instance Show Hash where
-    show hash = hashName hash
-instance Eq Hash where
-    h1 == h2 = hashName h1 == hashName h2 && hashSize h1 == hashSize h2
-
 -- | Cipher algorithm
 data Cipher = Cipher
     { cipherID           :: CipherID
@@ -89,7 +128,7 @@ data Cipher = Cipher
     }
 
 cipherKeyBlockSize :: Cipher -> Int
-cipherKeyBlockSize cipher = 2 * (hashSize (cipherHash cipher) + bulkIVSize bulk + bulkKeySize bulk)
+cipherKeyBlockSize cipher = 2 * (hashDigestSize (cipherHash cipher) + bulkIVSize bulk + bulkKeySize bulk)
   where bulk = cipherBulk cipher
 
 -- | Check if a specific 'Cipher' is allowed to be used
