@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 -- |
 -- Module      : Network.TLS.Context
 -- License     : BSD-style
@@ -41,7 +42,9 @@ module Network.TLS.Context
     , contextNew
     -- * Deprecated new contexts methods
     , contextNewOnHandle
+#ifdef INCLUDE_NETWORK
     , contextNewOnSocket
+#endif
 
     -- * Context hooks
     , contextHookSetHandshakeRecv
@@ -80,7 +83,9 @@ import Control.Monad.State
 import Data.IORef
 
 -- deprecated imports
+#ifdef INCLUDE_NETWORK
 import Network.Socket (Socket)
+#endif
 import System.IO (Handle)
 
 class TLSParams a where
@@ -194,6 +199,7 @@ contextNewOnHandle :: (MonadIO m, TLSParams params)
 contextNewOnHandle handle params = contextNew handle params
 {-# DEPRECATED contextNewOnHandle "use contextNew" #-}
 
+#ifdef INCLUDE_NETWORK
 -- | create a new context on a socket.
 contextNewOnSocket :: (MonadIO m, TLSParams params)
                    => Socket -- ^ Socket of the connection.
@@ -201,6 +207,7 @@ contextNewOnSocket :: (MonadIO m, TLSParams params)
                    -> m Context
 contextNewOnSocket sock params = contextNew sock params
 {-# DEPRECATED contextNewOnSocket "use contextNew" #-}
+#endif
 
 contextHookSetHandshakeRecv :: Context -> (Handshake -> IO Handshake) -> IO ()
 contextHookSetHandshakeRecv context f =
