@@ -18,6 +18,7 @@ import Network.TLS.Struct
 import Network.TLS.Cipher
 import Network.TLS.Compression
 import Network.TLS.Credentials
+import Network.TLS.Crypto.ECDH
 import Network.TLS.Extension
 import Network.TLS.Util (catchException, fromJust)
 import Network.TLS.IO
@@ -27,7 +28,8 @@ import Network.TLS.Handshake.State
 import Network.TLS.Handshake.Process
 import Network.TLS.Handshake.Key
 import Network.TLS.Measurement
-import Data.Maybe (isJust)
+import Data.Monoid
+import Data.Maybe (isJust, listToMaybe, mapMaybe)
 import Data.List (intersect, sortBy)
 import qualified Data.ByteString as B
 import Data.ByteString.Char8 ()
@@ -80,6 +82,12 @@ handshakeServer sparams ctx = liftIO $ do
 --
 handshakeServerWith :: ServerParams -> Context -> Handshake -> IO ()
 handshakeServerWith sparams ctx clientHello@(ClientHello clientVersion _ clientSession ciphers compressions exts _) = do
+    -- rejecting client initiated renegotiation to prevent DOS.
+    unless (supportedClientInitiatedRenegotiation (ctxSupported ctx)) $ do
+        established <- ctxEstablished ctx
+        eof <- ctxEOF ctx
+        when (established && not eof) $
+            throwCore $ Error_Protocol ("renegotiation is not allowed", False, NoRenegotiation)
     -- check if policy allow this new handshake to happens
     handshakeAuthorized <- withMeasure ctx (onNewHandshake $ serverHooks sparams)
     unless handshakeAuthorized (throwCore $ Error_HandshakePolicy "server: handshake denied")
@@ -89,6 +97,12 @@ handshakeServerWith sparams ctx clientHello@(ClientHello clientVersion _ clientS
     processHandshake ctx clientHello
 
     when (clientVersion == SSL2) $ throwCore $ Error_Protocol ("ssl2 is not supported", True, ProtocolVersion)
+    -- Fallback SCSV: RFC7507
+    -- TLS_FALLBACK_SCSV: {0x56, 0x00}
+    when (supportedFallbackScsv (ctxSupported ctx) &&
+          (0x5600 `elem` ciphers) &&
+          clientVersion /= maxBound) $
+        throwCore $ Error_Protocol ("fallback is not allowed", True, InappropriateFallback)
     chosenVersion <- case findHighestVersionFrom clientVersion (supportedVersions $ ctxSupported ctx) of
                         Nothing -> throwCore $ Error_Protocol ("client version " ++ show clientVersion ++ " is not supported", True, ProtocolVersion)
                         Just v  -> return v
@@ -98,20 +112,44 @@ handshakeServerWith sparams ctx clientHello@(ClientHello clientVersion _ clientS
     when (null commonCompressions) $ throwCore $
         Error_Protocol ("no compression in common with the client", True, HandshakeFailure)
 
+    let serverName = case extensionDecode False `fmap` (lookup extensionID_ServerName exts) of
+            Just (Just (ServerName ns)) -> listToMaybe (mapMaybe toHostName ns)
+                where toHostName (ServerNameHostName hostName) = Just hostName
+                      toHostName (ServerNameOther _)           = Nothing
+            _                           -> Nothing
+
     let ciphersFilteredVersion = filter (cipherAllowedForVersion chosenVersion) commonCiphers
         usedCipher = (onCipherChoosing $ serverHooks sparams) chosenVersion ciphersFilteredVersion
-        creds = sharedCredentials $ ctxShared ctx
+    extraCreds <- (onServerNameIndication $ serverHooks sparams) serverName
+    let creds = extraCreds `mappend` (sharedCredentials $ ctxShared ctx)
     cred <- case cipherKeyExchange usedCipher of
                 CipherKeyExchange_RSA     -> return $ credentialsFindForDecrypting creds
                 CipherKeyExchange_DH_Anon -> return $ Nothing
                 CipherKeyExchange_DHE_RSA -> return $ credentialsFindForSigning SignatureRSA creds
                 CipherKeyExchange_DHE_DSS -> return $ credentialsFindForSigning SignatureDSS creds
+                CipherKeyExchange_ECDHE_RSA -> return $ credentialsFindForSigning SignatureRSA creds
                 _                         -> throwCore $ Error_Protocol ("key exchange algorithm not implemented", True, HandshakeFailure)
 
     resumeSessionData <- case clientSession of
             (Session (Just clientSessionId)) -> liftIO $ sessionResume (sharedSessionManager $ ctxShared ctx) clientSessionId
             (Session Nothing)                -> return Nothing
 
+    maybe (return ()) (usingState_ ctx . setClientSNI) serverName
+
+    case extensionDecode False `fmap` (lookup extensionID_ApplicationLayerProtocolNegotiation exts) of
+        Just (Just (ApplicationLayerProtocolNegotiation protos)) -> usingState_ ctx $ setClientALPNSuggest protos
+        _ -> return ()
+
+    case extensionDecode False `fmap` (lookup extensionID_EllipticCurves exts) of
+        Just (Just (EllipticCurvesSupported es)) -> usingState_ ctx $ setClientEllipticCurveSuggest es
+        _ -> return ()
+
+    -- Currently, we don't send back EcPointFormats. In this case,
+    -- the client chooses EcPointFormat_Uncompressed.
+    case extensionDecode False `fmap` (lookup extensionID_EcPointFormats exts) of
+        Just (Just (EcPointFormatsSupported fs)) -> usingState_ ctx $ setClientEcPointFormatSuggest fs
+        _ -> return ()
+
     doHandshake sparams cred ctx chosenVersion usedCipher usedCompression clientSession resumeSessionData exts
 
   where
@@ -144,6 +182,38 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
     handshakeTerminate ctx
   where
         clientRequestedNPN = isJust $ lookup extensionID_NextProtocolNegotiation exts
+        clientALPNSuggest = isJust $ lookup extensionID_ApplicationLayerProtocolNegotiation exts
+
+        applicationProtocol = do
+            protos <- alpn
+            if null protos then npn else return protos
+
+        alpn | clientALPNSuggest = do
+            suggest <- usingState_ ctx $ getClientALPNSuggest
+            case (onALPNClientSuggest $ serverHooks sparams, suggest) of
+                (Just io, Just protos) -> do
+                    proto <- liftIO $ io protos
+                    usingState_ ctx $ do
+                        setExtensionALPN True
+                        setNegotiatedProtocol proto
+                    return $ [ ( extensionID_ApplicationLayerProtocolNegotiation
+                                                                                                               , extensionEncode $ ApplicationLayerProtocolNegotiation [proto]) ]
+                (_, _)                  -> return []
+             | otherwise = return []
+        npn = do
+            nextProtocols <-
+                if clientRequestedNPN
+                    then liftIO $ onSuggestNextProtocols $ serverHooks sparams
+                    else return Nothing
+            case nextProtocols of
+                Just protos -> do
+                    usingState_ ctx $ do
+                        setExtensionNPN True
+                        setServerNextProtocolSuggest protos
+                    return [ ( extensionID_NextProtocolNegotiation
+                             , extensionEncode $ NextProtocolNegotiation protos) ]
+                Nothing -> return []
+
 
         ---
         -- When the client sends a certificate, check whether
@@ -167,17 +237,8 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
                                     return $ extensionEncode (SecureRenegotiation cvf $ Just svf)
                             return [ (0xff01, vf) ]
                     else return []
-            nextProtocols <-
-                if clientRequestedNPN
-                    then liftIO $ onSuggestNextProtocols $ serverHooks sparams
-                    else return Nothing
-            npnExt <- case nextProtocols of
-                        Just protos -> do usingState_ ctx $ do setExtensionNPN True
-                                                               setServerNextProtocolSuggest protos
-                                          return [ ( extensionID_NextProtocolNegotiation
-                                                   , extensionEncode $ NextProtocolNegotiation protos) ]
-                        Nothing -> return []
-            let extensions = secRengExt ++ npnExt
+            protoExt <- applicationProtocol
+            let extensions = secRengExt ++ protoExt
             usingState_ ctx (setVersion chosenVersion)
             usingHState ctx $ setServerHelloParameters chosenVersion srand usedCipher usedCompression
             return $ ServerHello chosenVersion srand session (cipherID usedCipher)
@@ -198,6 +259,7 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
                         CipherKeyExchange_DH_Anon -> Just <$> generateSKX_DH_Anon
                         CipherKeyExchange_DHE_RSA -> Just <$> generateSKX_DHE SignatureRSA
                         CipherKeyExchange_DHE_DSS -> Just <$> generateSKX_DHE SignatureDSS
+                        CipherKeyExchange_ECDHE_RSA -> Just <$> generateSKX_ECDHE SignatureRSA
                         _                         -> return Nothing
             maybe (return ()) (sendPacket ctx . Handshake . (:[]) . ServerKeyXchg) skx
 
@@ -228,7 +290,7 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
             let dhparams = fromJust "server DHE Params" $ serverDHEParams sparams
             (priv, pub) <- generateDHE ctx dhparams
 
-            let serverParams = ServerDHParams dhparams pub
+            let serverParams = serverDHParamsFrom dhparams pub
 
             usingHState ctx $ setServerDHParams serverParams
             usingHState ctx $ modify $ \hst -> hst { hstDHPrivate = Just priv }
@@ -236,17 +298,7 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
 
         generateSKX_DHE sigAlg = do
             serverParams  <- setup_DHE
-            signatureData <- generateSignedDHParams ctx serverParams
-
-            usedVersion <- usingState_ ctx getVersion
-            let mhash = case usedVersion of
-                            TLS12 -> case filter ((==) sigAlg . snd) $ supportedHashSignatures $ ctxSupported ctx of
-                                          []  -> error ("no hash signature for " ++ show sigAlg)
-                                          x:_ -> Just (fst x)
-                            _     -> Nothing
-            let hashDescr = signatureHashData sigAlg mhash
-            signed <- signatureCreate ctx (fmap (\h -> (h, sigAlg)) mhash) hashDescr signatureData
-
+            signed <- digitallySignDHParams ctx serverParams sigAlg
             case sigAlg of
                 SignatureRSA -> return $ SKX_DHE_RSA serverParams signed
                 SignatureDSS -> return $ SKX_DHE_DSS serverParams signed
@@ -254,6 +306,31 @@ doHandshake sparams mcred ctx chosenVersion usedCipher usedCompression clientSes
 
         generateSKX_DH_Anon = SKX_DH_Anon <$> setup_DHE
 
+        setup_ECDHE curvename = do
+            let ecdhparams = ecdhParams curvename
+            (priv, pub) <- generateECDHE ctx ecdhparams
+
+            let serverParams = ServerECDHParams ecdhparams pub
+
+            usingHState ctx $ setServerECDHParams serverParams
+            usingHState ctx $ modify $ \hst -> hst { hstECDHPrivate = Just priv }
+            return (serverParams)
+
+        generateSKX_ECDHE sigAlg = do
+            ncs <- usingState_ ctx $ getClientEllipticCurveSuggest
+            let common = availableEllipticCurves `intersect` fromJust "ClientEllipticCurveSuggest" ncs
+                -- FIXME: Currently maximum strength is chosen.
+                --        There may be a better way to choose EC.
+                nc = if null common then error "No common EllipticCurves"
+                                    else maximum $ map fromEnumSafe16 common
+            serverParams <- setup_ECDHE nc
+            signed       <- digitallySignECDHParams ctx serverParams sigAlg
+            case sigAlg of
+                SignatureRSA -> return $ SKX_ECDHE_RSA serverParams signed
+                _            -> error ("generate skx_dhe unsupported signature type: " ++ show sigAlg)
+
+        -- create a DigitallySigned objects for DHParams or ECDHParams.
+
 -- | receive Client data in handshake until the Finished handshake.
 --
 --      <- [certificate]
@@ -303,11 +380,8 @@ recvClientData sparams ctx = runRecvState ctx (RecvStateHandshake processClientC
 
             usedVersion <- usingState_ ctx getVersion
             -- Fetch all handshake messages up to now.
-            msgs <- usingHState ctx $ B.concat <$> getHandshakeMessages
-            (hashMethod, toVerify) <- prepareCertificateVerifySignatureData ctx usedVersion mbHashSig msgs
-
-            -- Verify the signature.
-            verif <- signatureVerifyWithHashDescr ctx SignatureRSA hashMethod toVerify dsig
+            msgs  <- usingHState ctx $ B.concat <$> getHandshakeMessages
+            verif <- certificateVerifyCheck ctx usedVersion mbHashSig msgs dsig
 
             case verif of
                 True -> do
