@@ -291,17 +291,21 @@ decodeClientKeyXchg cp = -- case  ClientKeyXchg <$> (remaining >>= getBytes)
         parseCKE CipherKeyExchange_DHE_DSS = parseClientDHPublic
         parseCKE CipherKeyExchange_DH_Anon = parseClientDHPublic
         parseCKE CipherKeyExchange_ECDHE_RSA = parseClientECDHPublic
+        parseCKE CipherKeyExchange_ECDHE_ECDSA = parseClientECDHPublic
         parseCKE _                         = error "unsupported client key exchange type"
         parseClientDHPublic = CKX_DH . dhPublic <$> getInteger16
         parseClientECDHPublic = do
-            len <- getWord8
-            _ <- getWord8 -- Magic number 4
-            let siz = fromIntegral len `div` 2
-            xb <- getBytes siz
-            yb <- getBytes siz
-            let x = os2ip xb
-                y = os2ip yb
-            return $ CKX_ECDH $ ecdhPublic x y siz
+            len      <- getWord8
+            formatTy <- getWord8
+            case formatTy of
+                4 -> do -- uncompressed
+                    let siz = fromIntegral len `div` 2
+                    xb <- getBytes siz
+                    yb <- getBytes siz
+                    let x = os2ip xb
+                        y = os2ip yb
+                    return $ CKX_ECDH $ ecdhPublic x y siz
+                _ -> error ("unsupported EC format type: " ++ show formatTy)
 
 decodeServerKeyXchg_DH :: Get ServerDHParams
 decodeServerKeyXchg_DH = getServerDHParams
@@ -332,6 +336,10 @@ decodeServerKeyXchgAlgorithmData ver cke = toCKE
                 dhparams  <- getServerECDHParams
                 signature <- getDigitallySigned ver
                 return $ SKX_ECDHE_RSA dhparams signature
+            CipherKeyExchange_ECDHE_ECDSA -> do
+                dhparams  <- getServerECDHParams
+                signature <- getDigitallySigned ver
+                return $ SKX_ECDHE_ECDSA dhparams signature
             _ -> do
                 bs <- remaining >>= getBytes
                 return $ SKX_Unknown bs
@@ -395,6 +403,7 @@ encodeHandshakeContent (ServerKeyXchg skg) =
         SKX_DHE_RSA params sig -> putServerDHParams params >> putDigitallySigned sig
         SKX_DHE_DSS params sig -> putServerDHParams params >> putDigitallySigned sig
         SKX_ECDHE_RSA params sig -> putServerECDHParams params >> putDigitallySigned sig
+        SKX_ECDHE_ECDSA params sig -> putServerECDHParams params >> putDigitallySigned sig
         SKX_Unparsed bytes     -> putBytes bytes
         _                      -> error ("encodeHandshakeContent: cannot handle: " ++ show skg)
 
@@ -490,15 +499,29 @@ putServerDHParams (ServerDHParams p g y) = mapM_ putBigNum16 [p,g,y]
 
 getServerECDHParams :: Get ServerECDHParams
 getServerECDHParams = do
-    _ <- getWord8 -- ECParameters ECCurveType: curve name type, should be 3
-    w16 <- getWord16   -- ECParameters NamedCurve
-    mxy <- getOpaque16 -- ECPoint
-    let xy = B.drop 1 mxy
-        siz = B.length xy `div` 2
-        (xb,yb) = B.splitAt siz xy
-        x = os2ip xb
-        y = os2ip yb
-    return $ ServerECDHParams (ecdhParams w16) (ecdhPublic x y siz)
+    curveType <- getWord8
+    case curveType of
+        1 -> do -- explicit prime
+            _prime    <- getOpaque8
+            _a        <- getOpaque8
+            _b        <- getOpaque8
+            _base     <- getOpaque8
+            _order    <- getOpaque8
+            _cofactor <- getOpaque8
+            error "cannot handle explicit prime ECDH Params"
+        2 -> -- explicit_char2
+            error "cannot handle explicit char2 ECDH Params"
+        3 -> do -- ECParameters ECCurveType: curve name type
+            w16 <- getWord16   -- ECParameters NamedCurve
+            mxy <- getOpaque8 -- ECPoint
+            let xy = B.drop 1 mxy
+                siz = B.length xy `div` 2
+                (xb,yb) = B.splitAt siz xy
+                x = os2ip xb
+                y = os2ip yb
+            return $ ServerECDHParams (ecdhParams w16) (ecdhPublic x y siz)
+        _ ->
+            error "unknown type for ECDH Params"
 
 putServerECDHParams :: ServerECDHParams -> Put
 putServerECDHParams (ServerECDHParams ecdhparams ecdhpub) = do
