@@ -31,12 +31,22 @@ module Network.TLS.State
     , getSecureRenegotiation
     , setExtensionNPN
     , getExtensionNPN
+    , setExtensionALPN
+    , getExtensionALPN
     , setNegotiatedProtocol
     , getNegotiatedProtocol
     , setServerNextProtocolSuggest
     , getServerNextProtocolSuggest
+    , setClientALPNSuggest
+    , getClientALPNSuggest
+    , setClientEllipticCurveSuggest
+    , getClientEllipticCurveSuggest
+    , setClientEcPointFormatSuggest
+    , getClientEcPointFormatSuggest
     , getClientCertificateChain
     , setClientCertificateChain
+    , setClientSNI
+    , getClientSNI
     , getVerifiedData
     , setSession
     , getSession
@@ -52,12 +62,15 @@ import Network.TLS.Struct
 import Network.TLS.RNG
 import Network.TLS.Types (Role(..))
 import Network.TLS.Wire (GetContinuation)
+import Network.TLS.Extension
 import qualified Data.ByteString as B
 import Control.Monad.State
-import Control.Monad.Error
+import Network.TLS.ErrT
 import Crypto.Random
 import Data.X509 (CertificateChain)
 
+type HostName = String
+
 data TLSState = TLSState
     { stSession             :: Session
     , stSessionResuming     :: Bool
@@ -65,16 +78,21 @@ data TLSState = TLSState
     , stClientVerifiedData  :: Bytes -- RFC 5746
     , stServerVerifiedData  :: Bytes -- RFC 5746
     , stExtensionNPN        :: Bool  -- NPN draft extension
+    , stExtensionALPN       :: Bool  -- RFC 7301
     , stHandshakeRecordCont :: Maybe (GetContinuation (HandshakeType, Bytes))
-    , stNegotiatedProtocol  :: Maybe B.ByteString -- NPN protocol
+    , stNegotiatedProtocol  :: Maybe B.ByteString -- NPN and ALPN protocol
     , stServerNextProtocolSuggest :: Maybe [B.ByteString]
+    , stClientALPNSuggest   :: Maybe [B.ByteString]
+    , stClientEllipticCurveSuggest :: Maybe [NamedCurve]
+    , stClientEcPointFormatSuggest :: Maybe [EcPointFormat]
     , stClientCertificateChain :: Maybe CertificateChain
+    , stClientSNI           :: Maybe HostName
     , stRandomGen           :: StateRNG
     , stVersion             :: Maybe Version
     , stClientContext       :: Role
     }
 
-newtype TLSSt a = TLSSt { runTLSSt :: ErrorT TLSError (State TLSState) a }
+newtype TLSSt a = TLSSt { runTLSSt :: ErrT TLSError (State TLSState) a }
     deriving (Monad, MonadError TLSError, Functor, Applicative)
 
 instance MonadState TLSState TLSSt where
@@ -85,9 +103,9 @@ instance MonadState TLSState TLSSt where
 #endif
 
 runTLSState :: TLSSt a -> TLSState -> (Either TLSError a, TLSState)
-runTLSState f st = runState (runErrorT (runTLSSt f)) st
+runTLSState f st = runState (runErrT (runTLSSt f)) st
 
-newTLSState :: CPRG g => g -> Role -> TLSState
+newTLSState :: StateRNG -> Role -> TLSState
 newTLSState rng clientContext = TLSState
     { stSession             = Session Nothing
     , stSessionResuming     = False
@@ -95,11 +113,16 @@ newTLSState rng clientContext = TLSState
     , stClientVerifiedData  = B.empty
     , stServerVerifiedData  = B.empty
     , stExtensionNPN        = False
+    , stExtensionALPN       = False
     , stHandshakeRecordCont = Nothing
     , stNegotiatedProtocol  = Nothing
     , stServerNextProtocolSuggest = Nothing
+    , stClientALPNSuggest   = Nothing
+    , stClientEllipticCurveSuggest = Nothing
+    , stClientEcPointFormatSuggest = Nothing
     , stClientCertificateChain = Nothing
-    , stRandomGen           = StateRNG rng
+    , stClientSNI           = Nothing
+    , stRandomGen           = rng
     , stVersion             = Nothing
     , stClientContext       = clientContext
     }
@@ -179,6 +202,12 @@ setExtensionNPN b = modify (\st -> st { stExtensionNPN = b })
 getExtensionNPN :: TLSSt Bool
 getExtensionNPN = gets stExtensionNPN
 
+setExtensionALPN :: Bool -> TLSSt ()
+setExtensionALPN b = modify (\st -> st { stExtensionALPN = b })
+
+getExtensionALPN :: TLSSt Bool
+getExtensionALPN = gets stExtensionALPN
+
 setNegotiatedProtocol :: B.ByteString -> TLSSt ()
 setNegotiatedProtocol s = modify (\st -> st { stNegotiatedProtocol = Just s })
 
@@ -191,12 +220,36 @@ setServerNextProtocolSuggest ps = modify (\st -> st { stServerNextProtocolSugges
 getServerNextProtocolSuggest :: TLSSt (Maybe [B.ByteString])
 getServerNextProtocolSuggest = gets stServerNextProtocolSuggest
 
+setClientALPNSuggest :: [B.ByteString] -> TLSSt ()
+setClientALPNSuggest ps = modify (\st -> st { stClientALPNSuggest = Just ps})
+
+getClientALPNSuggest :: TLSSt (Maybe [B.ByteString])
+getClientALPNSuggest = gets stClientALPNSuggest
+
+setClientEllipticCurveSuggest :: [NamedCurve] -> TLSSt ()
+setClientEllipticCurveSuggest nc = modify (\st -> st { stClientEllipticCurveSuggest = Just nc})
+
+getClientEllipticCurveSuggest :: TLSSt (Maybe [NamedCurve])
+getClientEllipticCurveSuggest = gets stClientEllipticCurveSuggest
+
+setClientEcPointFormatSuggest :: [EcPointFormat] -> TLSSt ()
+setClientEcPointFormatSuggest epf = modify (\st -> st { stClientEcPointFormatSuggest = Just epf})
+
+getClientEcPointFormatSuggest :: TLSSt (Maybe [EcPointFormat])
+getClientEcPointFormatSuggest = gets stClientEcPointFormatSuggest
+
 setClientCertificateChain :: CertificateChain -> TLSSt ()
 setClientCertificateChain s = modify (\st -> st { stClientCertificateChain = Just s })
 
 getClientCertificateChain :: TLSSt (Maybe CertificateChain)
 getClientCertificateChain = gets stClientCertificateChain
 
+setClientSNI :: HostName -> TLSSt ()
+setClientSNI hn = modify (\st -> st { stClientSNI = Just hn })
+
+getClientSNI :: TLSSt (Maybe HostName)
+getClientSNI = gets stClientSNI
+
 getVerifiedData :: Role -> TLSSt Bytes
 getVerifiedData client = gets (if client == ClientRole then stClientVerifiedData else stServerVerifiedData)
 
@@ -205,11 +258,9 @@ isClientContext = gets stClientContext
 
 genRandom :: Int -> TLSSt Bytes
 genRandom n = do
-    st <- get
-    case withTLSRNG (stRandomGen st) (cprgGenerate n) of
-            (bytes, rng') -> put (st { stRandomGen = rng' }) >> return bytes
+    withRNG (getRandomBytes n)
 
-withRNG :: (forall g . CPRG g => g -> (a, g)) -> TLSSt a
+withRNG :: MonadPseudoRandom StateRNG a -> TLSSt a
 withRNG f = do
     st <- get
     let (a,rng') = withTLSRNG (stRandomGen st) f
