@@ -16,8 +16,8 @@ import Data.List (foldl')
 import Network.TLS.Struct (Bytes)
 import qualified Data.ByteString as B
 
-import Control.Exception (fromException)
-import qualified Control.Exception as E
+import Control.Exception (SomeException)
+import Control.Concurrent.Async
 
 sub :: Bytes -> Int -> Int -> Maybe Bytes
 sub b offset len
@@ -77,9 +77,5 @@ fmapEither f e = case e of
     Left l  -> Left l
     Right r -> Right (f r)
 
-catchException :: IO a -> (E.SomeException -> IO a) -> IO a
-catchException f handler = E.catchJust filterExn f handler
-  where filterExn :: E.SomeException -> Maybe E.SomeException
-        filterExn e = case fromException e of
-                            Just (_ :: E.AsyncException) -> Nothing
-                            Nothing                      -> Just e
+catchException :: IO a -> (SomeException -> IO a) -> IO a
+catchException action handler = withAsync action waitCatch >>= either handler return
