@@ -27,6 +27,7 @@ module Network.TLS.Wire
     , getOpaque16
     , getOpaque24
     , getInteger16
+    , getBigNum16
     , getList
     , processBytes
     , isEmpty
@@ -42,7 +43,9 @@ module Network.TLS.Wire
     , putOpaque16
     , putOpaque24
     , putInteger16
+    , putBigNum16
     , encodeWord16
+    , encodeWord32
     , encodeWord64
     ) where
 
@@ -50,7 +53,7 @@ import Data.Serialize.Get hiding (runGet)
 import qualified Data.Serialize.Get as G
 import Data.Serialize.Put
 import Control.Applicative ((<$>))
-import Control.Monad.Error
+import Control.Monad
 import qualified Data.ByteString as B
 import Data.Word
 import Data.Bits
@@ -113,6 +116,9 @@ getOpaque24 = getWord24 >>= getBytes
 getInteger16 :: Get Integer
 getInteger16 = os2ip <$> getOpaque16
 
+getBigNum16 :: Get BigNum
+getBigNum16 = BigNum <$> getOpaque16
+
 getList :: Int -> (Get (Int, a)) -> Get [a]
 getList totalLen getElement = isolate totalLen (getElements totalLen)
   where getElements len
@@ -131,6 +137,9 @@ putWords8 l = do
 putWord16 :: Word16 -> Put
 putWord16 = putWord16be
 
+putWord32 :: Word32 -> Put
+putWord32 = putWord32be
+
 putWords16 :: [Word16] -> Put
 putWords16 l = do
     putWord16 $ 2 * (fromIntegral $ length l)
@@ -158,8 +167,14 @@ putOpaque24 b = putWord24 (B.length b) >> putBytes b
 putInteger16 :: Integer -> Put
 putInteger16 = putOpaque16 . i2osp
 
+putBigNum16 :: BigNum -> Put
+putBigNum16 (BigNum b) = putOpaque16 b
+
 encodeWord16 :: Word16 -> Bytes
 encodeWord16 = runPut . putWord16
 
+encodeWord32 :: Word32 -> Bytes
+encodeWord32 = runPut . putWord32
+
 encodeWord64 :: Word64 -> Bytes
 encodeWord64 = runPut . putWord64be
